# -*- coding: utf-8 -*-
from rest_framework.response import Response
from rest_framework.views import APIView

from contentful_webhook_receiver.serializers import WebhookSerializer
from contentful_webhook_receiver.topics import topic_signal_mapping


class WebHookReceiverView(APIView):
    permission_classes = []

    def post(self, request, format=None):
        topic = request.headers.get('X-Contentful-Topic', '')
        serializer = WebhookSerializer(data={'data': request.data, 'type': topic})
        serializer.is_valid(raise_exception=True)
        instance = serializer.save()
        try:
            signal = topic_signal_mapping[topic]
            signal.send(self.__class__, instance=instance)
        except KeyError as e:
            raise Exception(f'Invalid Contentful topic: {topic}. Could not find a signal for this topic') from e
        return Response()



# -*- coding: utf-8 -*-

import uuid

from django.db import models


class WebhookInvocation(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    data = models.JSONField()
    type = models.CharField(max_length=255)
    updated_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ['-created_at']

    def __str__(self):
        return f'{self.created_at:%Y-%m-%d} | {self.type}'



from rest_framework import serializers

from contentful_webhook_receiver.models import WebhookInvocation


class WebhookSerializer(serializers.ModelSerializer):
    class Meta:
        model = WebhookInvocation
        fields = ['data', 'type']



# Generated by Django 3.1.6 on 2021-02-15 18:39

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='WebhookInvocation',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('data', models.JSONField()),
                ('type', models.CharField(max_length=255)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ['-created_at'],
            },
        ),
    ]



import django.dispatch


contentful_create_content_type = django.dispatch.Signal()
contentful_save_content_type = django.dispatch.Signal()
contentful_publish_content_type = django.dispatch.Signal()
contentful_unpublish_content_type = django.dispatch.Signal()
contentful_delete_content_type = django.dispatch.Signal()

contentful_create_entry = django.dispatch.Signal()
contentful_save_entry = django.dispatch.Signal()
contentful_autosave_entry = django.dispatch.Signal()
contentful_archive_entry = django.dispatch.Signal()
contentful_unarchive_entry = django.dispatch.Signal()
contentful_publish_entry = django.dispatch.Signal()
contentful_unpublish_entry = django.dispatch.Signal()
contentful_delete_entry = django.dispatch.Signal()

contentful_create_asset = django.dispatch.Signal()
contentful_save_asset = django.dispatch.Signal()
contentful_autosave_asset = django.dispatch.Signal()
contentful_archive_asset = django.dispatch.Signal()
contentful_unarchive_asset = django.dispatch.Signal()
contentful_publish_asset = django.dispatch.Signal()
contentful_unpublish_asset = django.dispatch.Signal()
contentful_delete_asset = django.dispatch.Signal()



from contentful_webhook_receiver.signals import contentful_publish_content_type, contentful_create_content_type, \
    contentful_save_content_type, contentful_delete_content_type, contentful_unpublish_content_type, \
    contentful_create_entry, contentful_save_entry, contentful_publish_entry, contentful_unpublish_entry, \
    contentful_delete_entry, contentful_unarchive_entry, contentful_archive_entry, contentful_autosave_entry, \
    contentful_create_asset, contentful_save_asset, contentful_autosave_asset, contentful_archive_asset, \
    contentful_unarchive_asset, contentful_publish_asset, contentful_unpublish_asset, contentful_delete_asset


class Topics:
    class ContentManagement:
        class ContentType:
            create = 'ContentManagement.ContentType.create'
            save = 'ContentManagement.ContentType.save'
            publish = 'ContentManagement.ContentType.publish'
            unpublish = 'ContentManagement.ContentType.unpublish'
            delete = 'ContentManagement.ContentType.delete'
        class Entry:
            create = 'ContentManagement.Entry.create'
            save = 'ContentManagement.Entry.save'
            autosave = 'ContentManagement.Entry.autosave'
            archive = 'ContentManagement.Entry.archive'
            unarchive = 'ContentManagement.Entry.unarchive'
            publish = 'ContentManagement.Entry.publish'
            unpublish = 'ContentManagement.Entry.unpublish'
            delete = 'ContentManagement.Entry.delete'
        class Asset:
            create = 'ContentManagement.Asset.create'
            save = 'ContentManagement.Asset.save'
            autosave = 'ContentManagement.Asset.autosave'
            archive = 'ContentManagement.Asset.archive'
            unarchive = 'ContentManagement.Asset.unarchive'
            publish = 'ContentManagement.Asset.publish'
            unpublish = 'ContentManagement.Asset.unpublish'
            delete = 'ContentManagement.Asset.delete'


topic_signal_mapping = {
    Topics.ContentManagement.ContentType.create : contentful_create_content_type,
    Topics.ContentManagement.ContentType.save : contentful_save_content_type,
    Topics.ContentManagement.ContentType.publish : contentful_publish_content_type,
    Topics.ContentManagement.ContentType.unpublish : contentful_unpublish_content_type,
    Topics.ContentManagement.ContentType.delete : contentful_delete_content_type,

    Topics.ContentManagement.Entry.create : contentful_create_entry,
    Topics.ContentManagement.Entry.save : contentful_save_entry,
    Topics.ContentManagement.Entry.autosave : contentful_autosave_entry,
    Topics.ContentManagement.Entry.archive : contentful_archive_entry,
    Topics.ContentManagement.Entry.unarchive : contentful_unarchive_entry,
    Topics.ContentManagement.Entry.publish : contentful_publish_entry,
    Topics.ContentManagement.Entry.unpublish : contentful_unpublish_entry,
    Topics.ContentManagement.Entry.delete : contentful_delete_entry,

    Topics.ContentManagement.Asset.create : contentful_create_asset,
    Topics.ContentManagement.Asset.save : contentful_save_asset,
    Topics.ContentManagement.Asset.autosave : contentful_autosave_asset,
    Topics.ContentManagement.Asset.archive : contentful_archive_asset,
    Topics.ContentManagement.Asset.unarchive : contentful_unarchive_asset,
    Topics.ContentManagement.Asset.publish : contentful_publish_asset,
    Topics.ContentManagement.Asset.unpublish : contentful_unpublish_asset,
    Topics.ContentManagement.Asset.delete : contentful_delete_asset,
}



# -*- coding: utf-8 -*-
from django.urls import path

from contentful_webhook_receiver.views import WebHookReceiverView

app_name = 'contentful_webhook_receiver'

urlpatterns = [
    path('contentful-webhook/', WebHookReceiverView.as_view()),
]
